% Code for analysis of pseudo-2D NMR Pro data
clear;
close all; clc; clf;

%Addpath for processing functions
addpath("C:\Users\Chekmenev Lab\Dropbox\XEUS DATA\MATLAB\gan_func");

%Short function designed to make whatever folder this script is run from the new working directory
%Useful if this is to be copied and executed in each experiment folder
scl

%% Variables

lb = 0.5;          %line broadening (Hz)
phase = 20;        %first order phase correction (deg)

%% Import thermal reference data from JCAMP-DX format

%Move to thermal reference data folder
cd 'Reference Data'

%Determine number of spectrum files in thermal experiment folder
m = length(dir) - 2;

%Tabulate all thermal spectrum filenames
tfiles = dir('*.dx');
for j = 1:m   
     filearray(j) = strcat(tfiles(j).folder + "\" + tfiles(j).name);
     thermJCAMPStruct{j} = jcampread(filearray(j));
end

for j = 1:m

thermjcampblock = thermJCAMPStruct{j}.Blocks;
thermtimedata = thermJCAMPStruct{j}.Blocks.XData; thermtimedata = (thermtimedata)';

if j == 1
thermrealdata = thermJCAMPStruct{j}.Blocks(1).YData; thermrealdata = (thermrealdata)';
thermimagdata = thermJCAMPStruct{j}.Blocks(2).YData; thermimagdata = (thermimagdata)';
elseif j >= 2
thermrealdata = j*thermJCAMPStruct{j}.Blocks(1).YData - (j-1)*thermJCAMPStruct{j-1}.Blocks(1).YData; thermrealdata = (thermrealdata)';
thermimagdata = j*thermJCAMPStruct{j}.Blocks(2).YData - (j-1)*thermJCAMPStruct{j-1}.Blocks(2).YData; thermimagdata = (thermimagdata)';
end

%Identify relevant acquisition paramaters from struct
thermjcampnotes = thermJCAMPStruct{j}.Notes;
O1Pt = str2double(thermjcampnotes(13,2));       %frequency offset for spectrum center
Ppmt = str2double(thermjcampnotes(8,2));        %frequency in MHz
dwellt = thermtimedata(2)-thermtimedata(1);     %dwell time (difference between time data)
swt = 1/dwellt;                                 %spectral width in Hz
swpt = swt/Ppmt;                                %convert sw to ppm

% [therm,thermdir] = uigetfile('*.*','Select thermal reference .dx spectrum file');
% cd(thermdir)
% %NOTE: jcampread function requires MATLAB Bioinformatics Toolbox
% ThermJCAMPStruct = jcampread(therm);
% %Alternative syntax for fixed-file reference
% %JCAMPStruct = jcampread('pablo_debug_0070_signal.dx');
% 
% %Identify time, real and imaginary vectors from struct
% thermjcampblock = ThermJCAMPStruct.Blocks;
% thermtimedata = thermjcampblock.XData; thermtimedata = (thermtimedata)';
% thermrealdata = thermjcampblock(1).YData; thermrealdata = (thermrealdata)';
% thermimagdata = thermjcampblock(2).YData; thermimagdata = (thermimagdata)';

%Define number of points * 4 for zero-filling
td = length(thermtimedata);
npt = td*4;            
%Format data into real and imaginary parts to perfom complex conjugation
fidt = thermrealdata + 1i*thermimagdata;

%Perform zero-filling, line-broadening and exp. multiplication
fidt = zf(fidt,npt);
fidt = em(fidt,lb,swt);

%Perform FFT of data and swap center of k-space
spectherm = swap(real(ph(fft(conj(fidt)),phase,0)));
axt = O1Pt+(swpt/2):-(swpt/npt):O1Pt-(swpt/2)+(swpt/npt);     %x-axis absolute frequency (ppm) 

%Baseline correct magnitude spectrum
specthermend = spectherm(round(npt*0.75):npt);
specthermbc = mean(specthermend);
spectherm = spectherm - specthermbc;

%Plot 1H magnitude spectrum
fig1 = plot(axt,spectherm,'b-');
set(gca,'XDir','rev')
axis([-2 10 -Inf Inf])      %set axis scale from -2 to 10 ppm
axis on;
xlabel('^1H Frequency (ppm)','FontSize',16,'FontWeight','bold')
ylabel('^1H NMR Magnitude (a.u.)','FontSize',16,'FontWeight','bold')
title('Thermal ^1H NMR Magnitude Spectrum from FID','FontSize',14,'FontWeight','bold')

cd ../
cd 'CSV Data'

%Write processed thermal spectral data to CSV format
axt = (axt)';
thermdatafull = [axt,spectherm];
thermtxt = num2str(j);
writematrix(thermdatafull,strcat('Thermal FFT Data -' + " " + thermtxt + '.csv'))

cd ../
cd 'PDF 1D Spectra'
%Save processed thermal spectrum to PDF format
thermname = strcat('Thermal FFT Data -' + " " + thermtxt);
print(thermname,'-dpdf')

%Calculate thermal reference integral from user boundaries
bound1 = 1271; bound2 = 1420;           %lower/upper intergration bounds
intdatat = spectherm(bound1:bound2);
intranget = axt(bound1:bound2);

%Perform trapezoidal integral of thermal 1H peak
Stherm(j) = trapz(intdatat);
[val, idx] = max(intdatat);
Stherm_max=max(intdatat);

cd ../
cd 'Reference Data'

end

%Calculate average thermal spectral integral from de-accumulated averages
Stherm_avg = mean(Stherm);

%% Import HP data from JCAMP-DX format

%Move to HP spectral data folder
cd ../
cd 'HP Data'

%Determine number of spectrum files in experiment folder
n = length(dir) - 2;

%Preallocate matrix sizes for speed
SHa = zeros(1,n);SHb = zeros(1,n);
EHa = zeros(1,n);EHb = zeros(1,n);
ExpDataALL = cell(length(n),10);

%Tabulate all spectrum filenames
files = dir('*.dx');
for k = 1:n   
     filearray(k) = strcat(files(k).folder + "\" + files(k).name);
     HPJCAMPStruct{k} = jcampread(filearray(k));
end

for k = 1:n  %change this to select the HP spectrum number you want to plot

HPjcampblock = HPJCAMPStruct{k}.Blocks;
HPtimedata = HPJCAMPStruct{k}.Blocks.XData; HPtimedata = (HPtimedata)';

if k == 1
HPrealdata = HPJCAMPStruct{k}.Blocks(1).YData; HPrealdata = (HPrealdata)';
HPimagdata = HPJCAMPStruct{k}.Blocks(2).YData; HPimagdata = (HPimagdata)';
elseif k >= 2
HPrealdata = k*HPJCAMPStruct{k}.Blocks(1).YData - (k-1)*HPJCAMPStruct{k-1}.Blocks(1).YData; HPrealdata = (HPrealdata)';
HPimagdata = k*HPJCAMPStruct{k}.Blocks(2).YData - (k-1)*HPJCAMPStruct{k-1}.Blocks(2).YData; HPimagdata = (HPimagdata)';
end

%Identify relevant acquisition paramaters from struct
HPjcampnotes = HPJCAMPStruct{k}.Notes;
O1PHP = str2double(HPjcampnotes(13,2));       %frequency offset for spectrum center
PpmHP = str2double(HPjcampnotes(8,2));        %frequency in MHz
dwellHP = HPtimedata(2)-HPtimedata(1);        %dwell time (difference between time data)
swHP = 1/dwellHP;                                 %spectral width in Hz
swpHP = swHP/PpmHP;                                %convert sw to ppm

%Define number of points * 4 for zero-filling
tdHP = length(HPtimedata);
npHP = td*4;            
%Format data into real and imaginary parts to perfom complex conjugation
fidHP = HPrealdata + 1i*HPimagdata;

%Perform zero-filling, line-broadening and exp. multiplication
fidHP = zf(fidHP,npHP);
fidHP = em(fidHP,lb,swHP);

%Perform FFT of data and swap center of k-space
specHP = swap(real(ph(fft(conj(fidHP)),phase,0)));
axHP = O1PHP+(swpHP/2):-(swpHP/npHP):O1PHP-(swpHP/2)+(swpHP/npHP);     %x-axis absolute frequency (ppm) 

%Baseline correct magnitude spectrum
specHPend = specHP(round(npHP*0.75):npHP);
specHPbc = mean(specHPend);
specHP = specHP - specHPbc;

%Plot 1H magnitude spectrum
figure(2)
fig2 = plot(axHP,specHP,'r-');
set(gca,'XDir','rev')
axis([-2 10 -Inf Inf])      %set axis scale from -2 to 10 ppm
axis on;
xlabel('^1H Frequency (ppm)','FontSize',16,'FontWeight','bold')
ylabel('^1H NMR Magnitude (a.u.)','FontSize',16,'FontWeight','bold')
title("HP ^1H NMR Magnitude Spectrum from FID - Exp #" + k,'FontSize',14,'FontWeight','bold')

%Write processed thermal spectral data to CSV format
axHP = (axHP)';
HPdatafull = [axHP,specHP];
HPtxt = num2str(k);

cd ../
cd 'CSV Data'
writematrix(HPdatafull,strcat('HP FFT Data -' + " " + HPtxt + '.csv'))

cd ../
cd 'PDF 1D Spectra'
%Save processed thermal spectrum to PDF format
HPname = strcat('HP FFT Data -' + " " + HPtxt);
print(HPname,'-dpdf')

%Calculate HP reference integral from user boundaries
bound3 = 1252; bound4 = 1354;           %lower/upper Ha intergration bounds
bound5 = 1354; bound6 = 1418;           %lower/upper Hb intergration bounds
intdataHa = specHP(bound3:bound4);
intdataHb = specHP(bound5:bound6);
intrangeHa = axt(bound3:bound4);
intrangeHb = axt(bound5:bound6);

%Perform trapezoidal integral of HP 1H peak
SHa(k) = trapz(intdataHa); 
SHb(k) = trapz(intdataHb);

%Calculate enhancement factor of HP 1H peak
EHa(k) = (SHa(k)/Stherm_avg)*8;
EHb(k) = (SHb(k)/Stherm_avg)*8;

%Print current experiment Ha/Hb enhancement values to Command Window9
fprintf('Exp. %1.0f - Ha Signal Enhancement = %4.3f, Hb Signal Enhancement = %4.3f\n',k,EHa(k),EHb(k))

cd ../
cd 'HP Data'

end

%Convert data to column arrays for tabulation
SHa = (SHa)'; SHb = (SHb)';
EHa = (EHa)'; EHb = (EHb)';

%% Plot signal enhancement vs. time

%Define experimental timestep
Duration = 33.47;                   %total scan duration in seconds
TimeStep = Duration/(n-1);          %time space between scans
Exptime = 0:TimeStep:Duration;      %creates linear array of time points  
Exptime = (Exptime)';

figure(3)
fig3 = plot(Exptime,EHa,'kx');
hold on
plot(Exptime,EHb,'rx')
axis on;
axis([0 Duration -Inf Inf])
xlabel('Time (seconds)','FontSize',16,'FontWeight','bold')
ylabel('^1H NMR Signal Enhancement Factor','FontSize',16,'FontWeight','bold')
title('HP Propane ^1H NMR Dynamic Run','FontSize',14,'FontWeight','bold')

cd ../
%Save processed thermal spectrum to PDF format
Expname = 'Exp Build and Decay';
print(Expname,'-dpdf')

%% Subplot T1 decay data

%Select bounds for fitting
bound7 = 43; bound8 = 58;
ExptimeT1 = Exptime(bound7:bound8);
EHaT1 = EHa(bound7:bound8);
EHbT1 = EHb(bound7:bound8);
t = length(ExptimeT1);

%Define Exp1 exponential decay fitting function
fHa = fit(ExptimeT1,EHaT1,'exp1');
fHb = fit(ExptimeT1,EHbT1,'exp1');

%Determine fitting coefficients, Pmax and uncertainty
coeffsHa = coeffvalues(fHa);
coeffsHb = coeffvalues(fHb);
% EHamax = coeffsHa(1);
confHa = confint(fHa); confHb = confint(fHb);

%Determine relaxation rate constants and uncertainty
T1valHa = abs(1/coeffsHa(2));
T1valHb = abs(1/coeffsHb(2));
xErrHa = abs(1/confHa(1,2)-(1/confHa(2,2)))/2;
xErrHb = abs(1/confHb(1,2)-(1/confHb(2,2)))/2;

%Create string arrays for labelling
T1valtxtHa = num2str(T1valHa);
T1valtxtHb = num2str(T1valHb);
xErrtxtHa = num2str(xErrHa);
xErrtxtHb = num2str(xErrHb);

%Concatenate strings to generate buildup & fit characteristics
fiteqtxt = 'y = a*exp(bx)';
figtxtHa = strcat('T_1(H_a) ='  + " " + T1valtxtHa + " " + '+/-' + " " + xErrtxtHa + " " + 'secs');
figtxtHb = strcat('T_1(H_b) ='  + " " + T1valtxtHb + " " + '+/-' + " " + xErrtxtHb + " " + 'secs');

%Create new plot for relaxation curve
figure(4)
plot(fHa,'k--',ExptimeT1,EHaT1,'ks');
hold on
plot(fHb,'r--',ExptimeT1,EHbT1,'rs');

axis on;
axis([ExptimeT1(1) ExptimeT1(t) -Inf Inf])
ax = gca; ax.FontSize = 14;
xlabel('Relaxation Time (secs)','FontSize',16,'FontWeight','bold')
ylabel('^1H NMR Signal Enhancement','FontSize',16,'FontWeight','bold')
title("^1H T_1 Relaxation",'FontSize',16,'FontWeight','bold')
legend('Location','NorthEastOutside')
xpos = ExptimeT1(6);
yposHa = (max(EHaT1)*0.8); yposHb = (min(EHbT1)*0.8);
text(xpos,yposHa,figtxtHa,'FontSize',14)
text(xpos,yposHb,figtxtHb,'Color','red','FontSize',14)
set(gcf,'Position',[100 100 750 500])

%Save snapshot of T1 data to PDF format
T1name = 'Exp T1 Decay';
print(T1name,'-dpdf')

%% Write all relevant data to file

%Assign thermal integral and T1 values & errors to master cell array
for StoreLoop = 1:n
    ExpDataALL{StoreLoop,1} = Exptime(StoreLoop);
    ExpDataALL{StoreLoop,2} = Stherm_avg;
    ExpDataALL{StoreLoop,3} = SHa(StoreLoop);
    ExpDataALL{StoreLoop,4} = EHa(StoreLoop);
    ExpDataALL{StoreLoop,5} = SHb(StoreLoop);
    ExpDataALL{StoreLoop,6} = EHb(StoreLoop);    
    ExpDataALL{StoreLoop,7} = T1valHa;
    ExpDataALL{StoreLoop,8} = xErrHa;
    ExpDataALL{StoreLoop,9} = T1valHb;
    ExpDataALL{StoreLoop,10} = xErrHb;    
end

%ExpDataALL = [Exptime,SthermTable,SHa,EHa,SHb,EHb,T1valHaTable,xErrHaTable,T1valHbTable,xErrHbTable];
ExpDataCell = cell2table(ExpDataALL,'VariableNames',{'Time' 'Stherm' 'SHa' 'EHa' 'SHb' 'EHb' 'T1 Ha' 'T1 Ha Err' 'T1 Hb' 'T1 Hb Err'});
writetable(ExpDataCell,'KPI.csv')